# Generated by Django 3.2 on 2022-04-06 23:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('unicef_security', '0002_auto_20201028_0154'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'user', 'verbose_name_plural': 'users'},
        ),
        migrations.DeleteModel(
            name='BusinessArea',
        ),
        migrations.DeleteModel(
            name='Region',
        ),
    ]
